/******************************************************************************
Author:         David Szybala
Website:        www.davidszybala.com
Alt Website:    www.twistedgerbil.com
Program:        light_linker.ms
Date:           05/23/11
Description:    This script provides an easy way to include/exclude objects
                from lights
******************************************************************************/

try (destroyDialog light_linker_rollout) catch()


struct LightLinker
(
    scene_lights = #(),
    
    /**************************************************************************
    Sorts items by name
    **************************************************************************/
    fn sort_by_name item_one item_two =
    (
        case of
        (
            (item_one.name < item_two.name): -1
            (item_one.name > item_two.name): 1
            default: 0
        ) --end case
    ), --end fn
    
    /**************************************************************************
    Collects all lights in the scene
    **************************************************************************/
    fn collect_scene_lights =
    (
        this.scene_lights = for current in lights where (superClassOf current) == light collect current
        
        if scene_lights.count > 0 do
            qsort this.scene_lights sort_by_name
    ), --end fn
    
    /**************************************************************************
    Returns true if the user light selection are valid nodes
    **************************************************************************/
    fn validate_light_selection light_indices =
    (
        for index in light_indices do
        (
            if not (IsValidNode this.scene_lights[index]) do
                return false
        ) --end for
        
        true --return value
    ), --end fn
    
    /**************************************************************************
    Sets the exclude list for the indexed lights
    **************************************************************************/
    fn set_exclude_list light_indices object_list keep_current =
    (
        for index in light_indices do
        (
            if keep_current and this.scene_lights[index].excludeList != undefined then
            (
                excluded = join (this.scene_lights[index].excludeList) object_list
                this.scene_lights[index].excludeList = excluded
            ) --end if
            else
                this.scene_lights[index].excludeList = object_list
        ) --end for
    ), --end fn
    
    /**************************************************************************
    Sets the include list for the indexed lights
    **************************************************************************/
    fn set_include_list light_indices object_list keep_current =
    (
        for index in light_indices do
        (
            if keep_current and this.scene_lights[index].includeList != undefined then
            (
                included = join (this.scene_lights[index].includeList) object_list
                this.scene_lights[index].includeList = included
            ) --end if
            else
                this.scene_lights[index].includeList = object_list
        ) --end for
    ), --end fn
    
    /**************************************************************************
    Sets the wirecolor for the indexed lights
    **************************************************************************/
    fn set_light_wirecolor light_indices light_color =
    (
        for index in light_indices do
        (
            this.scene_lights[index].wirecolor = light_color
            
            --vray lights have special wirecolor settings
            if isProperty this.scene_lights[index] "viewport_wire_color" do
            (
                this.scene_lights[index].viewport_wire_color_on = true
                this.scene_lights[index].viewport_wire_color = light_color
            ) --end if
        ) --end for
    ), --end fn
    
    /**************************************************************************
    Resets the wirecolor for the indexed lights
    **************************************************************************/
    fn reset_light_wirecolor light_indices =
    (
        for index in light_indices do
        (
            this.scene_lights[index].wirecolor = (color 255 229 0)
            
            --vray lights have special wirecolor settings
            if isProperty this.scene_lights[index] "viewport_wire_color" do
            (
                this.scene_lights[index].viewport_wire_color_on = false
                this.scene_lights[index].viewport_wire_color = (color 255 229 0)
            ) --end if
        ) --end for
    ), --end fn
    
    /**************************************************************************
    Resets the exclude/include list for the indexed lights
    **************************************************************************/
    fn reset_list light_indices =
    (
        for index in light_indices do
            this.scene_lights[index].excludeList = #()
    ), --end fn
    
    /**************************************************************************
    Returns all lights set to exclude objects. NOTE: If the light is set to
    exclude but has no objects it should NOT be selected
    **************************************************************************/
    fn get_exclude_lights =
    (
        local exclude_lights = #()
        
        for current in lights where (superClassOf current) == light do
        (
            if current.excludeList != undefined do
            (
                if current.excludeList.count > 0 do
                    append exclude_lights current
            ) --end if
        ) --end for
        
        exclude_lights --return value
    ), --end fn
    
    /**************************************************************************
    Returns all lights set to include objects. NOTE: If the light is set to
    include but has no objects it should be selected
    **************************************************************************/
    fn get_include_lights =
    (
        local include_lights = #()
        
        for current in lights where (superClassOf current) == light do
        (
            if current.includeList != undefined do
                append include_lights current
        ) --end for
        
        include_lights --return value
    ) --end fn
) --end struct


( --local scope
    global light_linker_rollout
    local light_linker_ini = getDir #plugcfg + "\\LightLinker.ini"
    local light_linker_position
    
    rollout light_linker_rollout "Light Linker"
    (
        local linker = LightLinker()
        
        /**********************************************************************
        Updates the light list
        **********************************************************************/
        fn update_light_list =
        (
            linker.collect_scene_lights()
            
            message = ("Lights: " + linker.scene_lights.count as string)
            light_linker_rollout.light_label.text = message
            
            current_lights = for current in linker.scene_lights collect current.name
            light_linker_rollout.light_list.items = current_lights
            
            message = "INFO: Light list updated"
            light_linker_rollout.status_label.text = message
        ) --end fn
        
        /**********************************************************************
        UI creation
        **********************************************************************/
        label light_label "Total: " pos:[5,5] width:150 height:15
        button update_button "Update" pos:[245,3] width:50 height:15
        MultiListBox light_list "" pos:[5,20] width:290 height:10
    
        groupBox mode_group "Mode" pos:[5,160] width:290 height:40
            radioButtons mode_radio "" pos:[15,180] width:140 height:16 labels:#("Exclude", "Include") columns:2
            checkbox preserve_checkbox "Preserve Current List" pos:[160,180] width:125 height:15
            
        button set_light_list_button "Exclude Selected Objects" pos:[5,210] width:290 height:20 enabled:false
        
        groupBox misc_group "Misc" pos:[5,235] width:290 height:70
            button set_color_button "Set Wirecolor" pos:[15,255] width:100 height:20 enabled:false
            colorPicker wirecolor_picker "" pos:[118,255] width:27 height:20 color:(color 5 255 255)
            button reset_color_button "Reset Wirecolor" pos:[155,255] width:130 height:20 enabled:false
            button reset_list_button "Reset 'Exclude' List" pos:[15,280] width:130 height:20 enabled:false
            button select_light_button "Select 'Exclude' Lights" pos:[155,280] width:130 height:20
        
        label status_label "INFO: Standing by..." pos:[5,310] width:290 height:15
        
        /**********************************************************************
        UI open event handler. Prepares the UI for first time open
        **********************************************************************/
        on light_linker_rollout open do
        (
            local current_color = execute (getIniSetting light_linker_ini "Dialog" "Wirecolor")
            
            if current_color == OK do
                current_color = color 5 255 255
            
            wirecolor_picker.color = current_color
            
            update_light_list()
        ) --end on
        
        /**********************************************************************
        UI move event handler. Stores the curent position of the UI
        **********************************************************************/
        on light_linker_rollout moved pos do
            setIniSetting light_linker_ini "Dialog" "Position" (pos as string)
        
        /**********************************************************************
        UI close event handler. Saves the current wirecolor for next open
        **********************************************************************/
        on light_linker_rollout close do
            setIniSetting light_linker_ini "Dialog" "Wirecolor" (wirecolor_picker.color as string)
        
        /**********************************************************************
        Update button click event handler. Updates the light list
        **********************************************************************/
        on update_button pressed do
            update_light_list()
        
        /**********************************************************************
        Light list selected event handler. Updates the UI based on the user
        list selection
        **********************************************************************/
        on light_list selected index do
        (
            local user_selection = light_list.selection as array
            local state = false
            
            if user_selection.count > 0 do state = true
            
            set_light_list_button.enabled = state
            set_color_button.enabled = state
            reset_color_button.enabled = state
            reset_list_button.enabled = state
        ) --end on
        
        /**********************************************************************
        Mode selected event handler. Updates the UI based on the user mode
        selection
        **********************************************************************/
        on mode_radio changed state do
        (
            if state == 1 then
            (
                preserve_checkbox.text = "Preserve Exclude List"
                set_light_list_button.text = "Exclude Selected Objects"
                reset_list_button.text = "Reset 'Exclude' List"
                select_light_button.text = "Select 'Exclude' Lights"
            ) --end if
            else
            (
                preserve_checkbox.text = "Preserve Include List"
                set_light_list_button.text = "Include Selected Objects"
                reset_list_button.text = "Reset 'Include' List"
                select_light_button.text = "Select 'Include' Lights"
            ) --end else
        ) --end on
        
        /**********************************************************************
        Set exclude/include button click event handler. Excludes/Includes the
        selected objects for the selected lights
        **********************************************************************/
        on set_light_list_button pressed do
        (
            local user_selection = light_list.selection as array
            
            if not linker.validate_light_selection user_selection do
            (
                status_label.text = "ERROR: Scene lights changed. Update list"
                return false
            ) --end if
                
            if selection.count == 0 do
            (
                status_label.text = "ERROR: No objects selected"
                return false
            ) --end if
                
            if mode_radio.state == 1 then
            (
                linker.set_exclude_list user_selection (selection as array) preserve_checkbox.checked
                status_label.text = "INFO: Selected objects have been excluded"
            ) --end if
            else
            (
                linker.set_include_list user_selection (selection as array) preserve_checkbox.checked
                status_label.text = "INFO: Selected objects have been included"
            ) --end else
        ) --end on
        
        /**********************************************************************
        Set wirecolor button click event handler. Sets the wirecolor for the
        selected lights
        **********************************************************************/
        on set_color_button pressed do
        (
            local user_selection = light_list.selection as array
            
            if not linker.validate_light_selection user_selection do
            (
                status_label.text = "ERROR: Scene lights changed. Update list"
                return false
            ) --end if
            
            linker.set_light_wirecolor user_selection wirecolor_picker.color
            
            status_label.text = "INFO: Light wirecolor set"
        ) --end on
        
        /**********************************************************************
        Reset wirecolor button click event handler. Resets the wirecolor for the
        selected lights
        **********************************************************************/
        on reset_color_button pressed do
        (
            local user_selection = light_list.selection as array
            
            if not linker.validate_light_selection user_selection do
            (
                status_label.text = "ERROR: Scene lights changed. Update list"
                return false
            ) --end if
            
            linker.reset_light_wirecolor user_selection
            
            status_label.text = "INFO: Light wirecolor reset"
        ) --end on
        
        /**********************************************************************
        Reset exclude/include list button click event handler. Resets the
        exclude/include list for the selected lights
        **********************************************************************/
        on reset_list_button pressed do
        (
            local user_selection = light_list.selection as array
            
            if not linker.validate_light_selection user_selection do
            (
                status_label.text = "ERROR: Scene lights changed. Update list"
                return false
            ) --end if
            
            linker.reset_list user_selection
            
            status_label.text = "INFO: Exclude/Include list reset"
        ) --end on
        
        -------------------------------------------------------------------
        on select_light_button pressed do
        (
            local select_lights = #()
            local message = ""
            
            if mode_radio.state == 1 then
            (
                select_lights = linker.get_exclude_lights()
                message = "exclude"
            ) --end if
            else
            (
                select_lights = linker.get_include_lights()
                message = "include"
            ) --end else
            
            clearSelection()
            
            if select_lights.count > 0 do
                select select_lights
            
            message = ("INFO: " + (select_lights.count as string) + " " + message + " found")
            status_label.text = message
        ) --end on
    ) --end rollout
    
    light_linker_position = execute (getIniSetting light_linker_ini "Dialog" "Position")
    if light_linker_position == OK do light_linker_position = [100,100]
    if light_linker_position.x > sysinfo.desktopsize.x - 300 do
        light_linker_position.x = sysinfo.desktopsize.x - 305
    if light_linker_position.y > sysinfo.desktopsize.y - 385 do
        light_linker_position.y = sysinfo.desktopsize.y - 385
    if light_linker_position.x < 0 do light_linker_position.x = 0 
    if light_linker_position.y < 0 do light_linker_position.y = 0
    createDialog light_linker_rollout 300 325 light_linker_position.x light_linker_position.y
) --end local scope